/*===---- bcd.h - Standard header for Binary-Coded Decimal Function ---===
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 *===-----------------------------------------------------------------------===
 */
 
#ifndef __BCD_H
#define __BCD_H

typedef vector unsigned char bcd;

#define BCD_FORMAT_IBM 0
#define BCD_FORMAT_Z 0
#define BCD_FORMAT_POWER 0
#define BCD_FORMAT_IBMi 1
#define BCD_FORMAT_I 1
#define BCD_FORMAT_NCR 1

#ifndef bcd_default_format
#define bcd_default_format __BCD_FORMAT_IBM
#endif

#include <altivec.h>

#define bcd_add(a,b) ((bcd)__builtin_bcdadd (a,b,bcd_default_format))
#define bcd_sub(A,b) ((bcd)__builtin_bcdsub (a,b,bcd_default_format))
#define bcd_add_ofl(a,b) ((_Bool)__builtin_bcdadd_ofl (a,b))
#define bcd_sub_ofl(a,b) ((_Bool)__builtin_bcdsub_ofl (a,b))
#define bcd_invalid(a) ((_Bool)__builtin_bcd_invalid (a))
#define bcd_cmpeq(a,b) ((_Bool)__builtin_bcdcmpeq (a,b))
#define bcd_cmpge(a,b) ((_Bool)__builtin_bcdcmpge (a,b))
#define bcd_cmpgt(a,b) ((_Bool)__builtin_bcdcmpgt (a,b))
#define bcd_cmple(a,b) ((_Bool)__builtin_bcdcmple (a,b))
#define bcd_cmplt(a,b) ((_Bool)__builtin_bcdcmplt (a,b))
#define bcd_cmpne(a,b) (!(_Bool)__builtin_bcdcmpeq (a,b))
#define bcd_xl(a,b) ((bcd)vec_xl_len_r(a,b))
#define bcd_xst(a,b) ((bcd)vec_xst_len_r(a,b))
#define bcd_quantize(d) (__builtin_bcdquantize(d))
#define bcd_dfp(a) (__builtin_bcd2dfp (a))
#define bcd_dfp2bcd(DFP) ((bcd)__builtin_vec_DFP2BCD (_Decimal128 dfp))
#define bcd_string2bcd(string) ((bcd) __bcd_string2bcd (string, bcd_default_format)
#define bcd_mul10(a) ((bcd) __builtin_bcdmul10 (a))
#define bcd_div10(a) ((bcd) __builtin_bcddiv10 (a))
#define bcd_mul(a,b) ((bcd) __bcd_mul (a,b,bcd_default_format))
#define bcd_div(a,b) ((bcd) __bcd_div (a,b,bcd_default_format))

vector unsigned char __builtin_bcdadd (vector unsigned char, vector unsigned char, const int) __asm__("__IBMBuiltin___builtin_bcdadd_VUcVUcVUciC");
vector unsigned char __builtin_bcdcopysign (vector unsigned char, vector unsigned char) __asm__("__IBMBuiltin___builtin_bcdcopysign_VUcVUcVUc");
vector unsigned char __builtin_bcdmul10 (vector unsigned char) __asm__("__IBMBuiltin___builtin_bcdmul10_VUcVUc");
vector unsigned char __builtin_bcdmul10c (vector unsigned char) __asm__("__IBMBuiltin___builtin_bcdmul10c_VUcVUc");
vector unsigned char __builtin_bcdmul10e (vector unsigned char, vector unsigned char) __asm__("__IBMBuiltin___builtin_bcdmul10e_VUcVUcVUc");
vector unsigned char __builtin_bcdmul10ec (vector unsigned char, vector unsigned char) __asm__("__IBMBuiltin___builtin_bcdmul10ec_VUcVUcVUc");
vector unsigned char __builtin_bcdsetsign (vector unsigned char, unsigned char) __asm__("__IBMBuiltin___builtin_bcdsetsign_VUcVUcUc");
vector unsigned char __builtin_bcdshift (vector unsigned char, int, unsigned char) __asm__("__IBMBuiltin___builtin_bcdshift_VUcVUciUc");
vector unsigned char __builtin_bcdshiftround (vector unsigned char, int, unsigned char) __asm__("__IBMBuiltin___builtin_bcdshiftround_VUcVUciUc");
vector unsigned char __builtin_bcdtruncate (vector unsigned char, int, unsigned char) __asm__("__IBMBuiltin___builtin_bcdtruncate_VUcVUciUc");
vector unsigned char __builtin_bcdunsignedshift (vector unsigned char, int) __asm__("__IBMBuiltin___builtin_bcdunsignedshift_VUcVUci");
vector unsigned char __builtin_bcdunsignedtruncate (vector unsigned char, int) __asm__("__IBMBuiltin___builtin_bcdunsignedtruncate_VUcVUci");
vector unsigned char __builtin_national2packed (vector unsigned char, unsigned char) __asm__("__IBMBuiltin___builtin_national2packed_VUcVUcUc");
vector unsigned char __builtin_packed2national (vector unsigned char) __asm__("__IBMBuiltin___builtin_packed2national_VUcVUc");
vector unsigned char __builtin_packed2zoned (vector unsigned char, unsigned char) __asm__("__IBMBuiltin___builtin_packed2zoned_VUcVUcUc");
vector unsigned char __builtin_zoned2packed (vector unsigned char, unsigned char) __asm__("__IBMBuiltin___builtin_zoned2packed_VUcVUcUc");

#endif /* __BCD_H */
